
/*
    Copyright 2004 Kevin dot Hammack at comcast dot net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 */

#include<glib.h>
#include<stdio.h>
#include<string.h>
#include<sys/types.h>
#include<fcntl.h>

#define TEST 

/*
  <warlord> Yea, just returning a GList would be fine..  Or a GList of
  GLists, one per row..  
*/


/* 
   split_csv_list - Take a string, split on commas into a GList.  
   Tricky part: honor quotes.  
   Quotes aren't stripped.  
   This would have been both easier and cleaner in scheme, and / or
   using regular expressions.  
   I was thinking small and fast when I wrote it. :-P
*/

GList *
split_csv_line(char *line) {
     GList *csvlist = NULL;
     gchar *begin;
     gchar *current;
     gchar *cell;
     gchar quote=0;
     gboolean eol = FALSE;

     current = line;
     begin = current;

     while (!eol) {
	  
	  if (quote && 
	      (*current == quote) && (*(current-1) != '\\') && 
	      (current != begin)) {

	       quote = 0;
	  }
	  else if (!quote && (*current == '"')) { quote = '"'; }
	  else if (!quote && (*current == '\'')) { quote = '\''; }

	  if (!quote && (*current == ',')) { *current = 0; }

	  if (*current == '\n') {
	       *current = 0;
	       eol = TRUE;
	  }

	  if (*current == 0) {
	       cell = strdup( begin );
	       csvlist = g_list_append(csvlist, cell);
	       current++;
	       begin = current;
	       quote = 0;
	  }
	  else {
	       current++;
	  }

     }

     return csvlist;
}



GList *
parse_csv (FILE *handle) 
{
     GList *csvlists = NULL;
     ssize_t bytes_read;
     char *line;
     size_t len = 0;
    
     while (bytes_read = getline (&line, &len, handle) != -1) {

	  csvlists = g_list_append(csvlists, split_csv_line(line));
	  free(line);
	  len=0;
     }

     return csvlists;
}



#ifdef TEST

void print_glist_rec (GList *list) {

}
//     print_glist_rec(list);

void print_glist(GList *list, gpointer dummy) {

     printf("(");
     
     while (list != NULL) {  
	  if (((*list).data != NULL) && ( *((char *) list->data) != 0)) {
	       printf( "%s",  (*list).data) ;
	  }
	  else {
	       printf( "\"\"" );
	  }
	  list = (*list).next;
	  if (list) {
	       printf(" ");
	  }
     }

     printf(")\n");
}
 
int main (int argc, char **argv) {

     FILE *fp;
     int result;
     GList *parsed_csv;
     GList *current;
     int dummy = 1;

     if (argc < 2) {
	  printf("usage:\n\tcsv2glist fname.csv\n");
     }

     fp = fopen (argv[1], "r");
     if (fp == NULL) return 1;

     parsed_csv = parse_csv(fp);

     g_list_foreach(parsed_csv, (GFunc)print_glist, &dummy);

}

#endif
